 /*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.teststeps;

import javax.swing.ImageIcon;

import org.apache.xmlbeans.XmlObject;

import com.eviware.soapui.config.RequestAssertionConfig;
import com.eviware.soapui.impl.wsdl.submit.WsdlMessageExchange;
import com.eviware.soapui.impl.wsdl.support.assertions.Assertable;
import com.eviware.soapui.impl.wsdl.support.assertions.Assertable.AssertionStatus;
import com.eviware.soapui.impl.wsdl.teststeps.assertions.AssertionError;
import com.eviware.soapui.impl.wsdl.teststeps.assertions.AssertionException;
import com.eviware.soapui.impl.wsdl.teststeps.assertions.WsdlAssertionRegistry;
import com.eviware.soapui.model.iface.SubmitContext;
import com.eviware.soapui.model.settings.Settings;
import com.eviware.soapui.model.support.AbstractModelItem;
import com.eviware.soapui.support.UISupport;

/**
 * Base class for WsdlAssertions
 * 
 * @author Ole.Matzura
 */

public abstract class WsdlMessageAssertion extends AbstractModelItem
{
	private RequestAssertionConfig assertionConfig;
   private final Assertable assertable;
   private AssertionStatus assertionStatus = AssertionStatus.UNKNOWN;
   private com.eviware.soapui.impl.wsdl.teststeps.assertions.AssertionError [] assertionErrors;
   private ImageIcon validIcon;
   private ImageIcon failedIcon;
   private ImageIcon unknownIcon;
   
   public final static String STATUS_PROPERTY = WsdlMessageAssertion.class.getName() + "@status";  
   public final static String ERRORS_PROPERTY = WsdlMessageAssertion.class.getName() + "@errors";  
   public final static String CONFIGURATION_PROPERTY = WsdlMessageAssertion.class.getName() + "@configuration";
	private final boolean cloneable;
	private final boolean configurable;
	private final boolean allowMultiple;  
   
   protected WsdlMessageAssertion(RequestAssertionConfig assertionConfig, Assertable modelItem, 
   			boolean cloneable, boolean configurable, boolean multiple )
   {
      this.assertionConfig = assertionConfig;
      this.assertable = modelItem;
		this.cloneable = cloneable;
		this.configurable = configurable;
		this.allowMultiple = multiple;
      
      validIcon = UISupport.createImageIcon("/valid_assertion.gif");
      failedIcon = UISupport.createImageIcon("/failed_assertion.gif");
      unknownIcon = UISupport.createImageIcon("/unknown_assertion.gif");      
   }
   
   public XmlObject getConfiguration()
   {
      if( null == assertionConfig.getConfiguration())
      {
         assertionConfig.addNewConfiguration();
      }
      
      return assertionConfig.getConfiguration();
   }

   public void setConfiguration( XmlObject configuration )
   {
	   XmlObject oldConfig = assertionConfig.getConfiguration();
      assertionConfig.setConfiguration( configuration );
      notifyPropertyChanged( WsdlMessageAssertion.CONFIGURATION_PROPERTY, oldConfig, configuration );
   }
   
   public String getName()
   {
      return assertionConfig.isSetName() ? assertionConfig.getName() : 
      	WsdlAssertionRegistry.getInstance().getAssertionNameForType( assertionConfig.getType());
   }

   public void setName(String name)
   {
      String old = getName();
      assertionConfig.setName( name );
      notifyPropertyChanged( NAME_PROPERTY, old, name );
   }
   
   public AssertionStatus getStatus()
   {
      return assertionStatus;
   }

   public AssertionError[] getErrors()
   {
      return assertionErrors;
   }

   public boolean isAllowMultiple()
   {
   	return allowMultiple;
   }
   
   public AssertionStatus assertResponse( WsdlMessageExchange messageExchange, SubmitContext context)
   {
      AssertionStatus oldStatus = assertionStatus;
      AssertionError[] oldErrors = getErrors();
      ImageIcon oldIcon = getIcon();
      
      if( !messageExchange.hasResponse() )
      {
      	if( messageExchange.getOperation().isOneWay() )
      	{
      		assertionStatus = AssertionStatus.VALID;
	         assertionErrors = null;
      	}
      	else
      	{
      		assertionStatus = AssertionStatus.FAILED;
      		assertionErrors = new com.eviware.soapui.impl.wsdl.teststeps.assertions.AssertionError[] 
      	                    { new com.eviware.soapui.impl.wsdl.teststeps.assertions.AssertionError("null/empty response") };
      	}
      }
      else
      {
	      try
	      {
	      	internalAssertResponse( messageExchange, context );
	         assertionStatus = AssertionStatus.VALID;
	         assertionErrors = null;
	      }
	      catch ( AssertionException e )
	      {
	      	assertionStatus = AssertionStatus.FAILED;
	      	assertionErrors = e.getErrors();
	      }
	      catch (Throwable e)
	      {
	         assertionStatus = AssertionStatus.FAILED;
	         assertionErrors = new com.eviware.soapui.impl.wsdl.teststeps.assertions.AssertionError[] 
                           { new com.eviware.soapui.impl.wsdl.teststeps.assertions.AssertionError( e.getMessage() )};
	      }
      }
      
      notifyPropertyChanged( STATUS_PROPERTY, oldStatus, assertionStatus );
      notifyPropertyChanged( ERRORS_PROPERTY, oldErrors, assertionErrors );
      notifyPropertyChanged( ICON_PROPERTY, oldIcon, getIcon() );
      
      return assertionStatus;
   }

   protected abstract String internalAssertResponse(WsdlMessageExchange messageExchange, SubmitContext context ) throws AssertionException;
   
   public AssertionStatus assertRequest( WsdlMessageExchange messageExchange, SubmitContext context)
   {
      AssertionStatus oldStatus = assertionStatus;
      ImageIcon oldIcon = getIcon();
      
      if( !messageExchange.hasRequest( true ) )
      {
      	assertionStatus = AssertionStatus.FAILED;
      	assertionErrors = new com.eviware.soapui.impl.wsdl.teststeps.assertions.AssertionError[] 
      	                    { new com.eviware.soapui.impl.wsdl.teststeps.assertions.AssertionError("null/empty response") };
      }
      else
      {
	      try
	      {
	      	internalAssertRequest( messageExchange, context );
	         assertionStatus = AssertionStatus.VALID;
	         assertionErrors = null;
	      }
	      catch ( AssertionException e )
	      {
	      	assertionStatus = AssertionStatus.FAILED;
	      	assertionErrors = e.getErrors();
	      }
	      catch (Throwable e)
	      {
	         assertionStatus = AssertionStatus.FAILED;
	         assertionErrors = new com.eviware.soapui.impl.wsdl.teststeps.assertions.AssertionError[] 
                           { new com.eviware.soapui.impl.wsdl.teststeps.assertions.AssertionError( e.getMessage() )};
	      }
      }
      
      notifyPropertyChanged( STATUS_PROPERTY, oldStatus, assertionStatus );
      notifyPropertyChanged( ICON_PROPERTY, oldIcon, getIcon() );
      
      return assertionStatus;
   }
   
   protected abstract String internalAssertRequest(WsdlMessageExchange messageExchange, SubmitContext context ) throws AssertionException;

   public boolean isConfigurable()
   {
      return configurable;
   }
   
   public boolean isClonable()
   {
      return cloneable;
   }
   
   public boolean configure()
   {
   	return true;
   }

   public String getDescription()
	{
		return getConfig().getDescription();
	}

	public ImageIcon getIcon()
   {
      switch( getStatus() )
      {
         case FAILED : return failedIcon; 
         case UNKNOWN : return unknownIcon; 
         case VALID : return validIcon;
      }
      
      return null;
   }

	public void updateConfig(RequestAssertionConfig config)
	{
		this.assertionConfig = config;
	}

	public RequestAssertionConfig getConfig()
	{
		return assertionConfig;
	}
	
	public Settings getSettings()
	{
		return assertable.getTestStep().getSettings();
	}

	public void release()
	{
	}
	
	public Assertable getAssertable()
	{
		return assertable;
	}
}

  